""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class AutoNodeMoveParams(CoreObject):
    """Parameters used to improve volume mesh by auto node move.
    """
    _default_params = {}

    def __initialize(
            self,
            quality_measure: CellQualityMeasure,
            target_quality: float,
            dihedral_angle: float,
            n_iterations_per_node: int,
            restrict_boundary_nodes_along_surface: bool,
            n_attempts: int):
        self._quality_measure = CellQualityMeasure(quality_measure)
        self._target_quality = target_quality
        self._dihedral_angle = dihedral_angle
        self._n_iterations_per_node = n_iterations_per_node
        self._restrict_boundary_nodes_along_surface = restrict_boundary_nodes_along_surface
        self._n_attempts = n_attempts

    def __init__(
            self,
            model: CommunicationManager=None,
            quality_measure: CellQualityMeasure = None,
            target_quality: float = None,
            dihedral_angle: float = None,
            n_iterations_per_node: int = None,
            restrict_boundary_nodes_along_surface: bool = None,
            n_attempts: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the AutoNodeMoveParams.

        Parameters
        ----------
        model: Model
            Model to create a AutoNodeMoveParams object with default parameters.
        quality_measure: CellQualityMeasure, optional
            Specify cell quality measure to be used for volume mesh improvement. The default value for cell quality measure is skewness.
        target_quality: float, optional
            Specify target quality used for the mesh improvement based on specified quality measure.
        dihedral_angle: float, optional
            Dihedral angle used to mantain features of boundary face zonelets.
        n_iterations_per_node: int, optional
            Number of iterations per node to be moved.
        restrict_boundary_nodes_along_surface: bool, optional
            Option to restrict the movement of the boundary node to the plane containing the boundary faces sharing the boundary node.
        n_attempts: int, optional
            Number of attempts to improve specified quality measure by node movement.
        json_data: dict, optional
            JSON dictionary to create a AutoNodeMoveParams object with provided parameters.

        Examples
        --------
        >>> auto_node_move_params = prime.AutoNodeMoveParams(model = model)
        """
        if json_data:
            self.__initialize(
                CellQualityMeasure(json_data["qualityMeasure"] if "qualityMeasure" in json_data else None),
                json_data["targetQuality"] if "targetQuality" in json_data else None,
                json_data["dihedralAngle"] if "dihedralAngle" in json_data else None,
                json_data["nIterationsPerNode"] if "nIterationsPerNode" in json_data else None,
                json_data["restrictBoundaryNodesAlongSurface"] if "restrictBoundaryNodesAlongSurface" in json_data else None,
                json_data["nAttempts"] if "nAttempts" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [quality_measure, target_quality, dihedral_angle, n_iterations_per_node, restrict_boundary_nodes_along_surface, n_attempts])
            if all_field_specified:
                self.__initialize(
                    quality_measure,
                    target_quality,
                    dihedral_angle,
                    n_iterations_per_node,
                    restrict_boundary_nodes_along_surface,
                    n_attempts)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "AutoNodeMoveParams")
                    json_data = param_json["AutoNodeMoveParams"] if "AutoNodeMoveParams" in param_json else {}
                    self.__initialize(
                        quality_measure if quality_measure is not None else ( AutoNodeMoveParams._default_params["quality_measure"] if "quality_measure" in AutoNodeMoveParams._default_params else CellQualityMeasure(json_data["qualityMeasure"] if "qualityMeasure" in json_data else None)),
                        target_quality if target_quality is not None else ( AutoNodeMoveParams._default_params["target_quality"] if "target_quality" in AutoNodeMoveParams._default_params else (json_data["targetQuality"] if "targetQuality" in json_data else None)),
                        dihedral_angle if dihedral_angle is not None else ( AutoNodeMoveParams._default_params["dihedral_angle"] if "dihedral_angle" in AutoNodeMoveParams._default_params else (json_data["dihedralAngle"] if "dihedralAngle" in json_data else None)),
                        n_iterations_per_node if n_iterations_per_node is not None else ( AutoNodeMoveParams._default_params["n_iterations_per_node"] if "n_iterations_per_node" in AutoNodeMoveParams._default_params else (json_data["nIterationsPerNode"] if "nIterationsPerNode" in json_data else None)),
                        restrict_boundary_nodes_along_surface if restrict_boundary_nodes_along_surface is not None else ( AutoNodeMoveParams._default_params["restrict_boundary_nodes_along_surface"] if "restrict_boundary_nodes_along_surface" in AutoNodeMoveParams._default_params else (json_data["restrictBoundaryNodesAlongSurface"] if "restrictBoundaryNodesAlongSurface" in json_data else None)),
                        n_attempts if n_attempts is not None else ( AutoNodeMoveParams._default_params["n_attempts"] if "n_attempts" in AutoNodeMoveParams._default_params else (json_data["nAttempts"] if "nAttempts" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            quality_measure: CellQualityMeasure = None,
            target_quality: float = None,
            dihedral_angle: float = None,
            n_iterations_per_node: int = None,
            restrict_boundary_nodes_along_surface: bool = None,
            n_attempts: int = None):
        """Set the default values of AutoNodeMoveParams.

        Parameters
        ----------
        quality_measure: CellQualityMeasure, optional
            Specify cell quality measure to be used for volume mesh improvement. The default value for cell quality measure is skewness.
        target_quality: float, optional
            Specify target quality used for the mesh improvement based on specified quality measure.
        dihedral_angle: float, optional
            Dihedral angle used to mantain features of boundary face zonelets.
        n_iterations_per_node: int, optional
            Number of iterations per node to be moved.
        restrict_boundary_nodes_along_surface: bool, optional
            Option to restrict the movement of the boundary node to the plane containing the boundary faces sharing the boundary node.
        n_attempts: int, optional
            Number of attempts to improve specified quality measure by node movement.
        """
        args = locals()
        [AutoNodeMoveParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of AutoNodeMoveParams.

        Examples
        --------
        >>> AutoNodeMoveParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in AutoNodeMoveParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._quality_measure is not None:
            json_data["qualityMeasure"] = self._quality_measure
        if self._target_quality is not None:
            json_data["targetQuality"] = self._target_quality
        if self._dihedral_angle is not None:
            json_data["dihedralAngle"] = self._dihedral_angle
        if self._n_iterations_per_node is not None:
            json_data["nIterationsPerNode"] = self._n_iterations_per_node
        if self._restrict_boundary_nodes_along_surface is not None:
            json_data["restrictBoundaryNodesAlongSurface"] = self._restrict_boundary_nodes_along_surface
        if self._n_attempts is not None:
            json_data["nAttempts"] = self._n_attempts
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "quality_measure :  %s\ntarget_quality :  %s\ndihedral_angle :  %s\nn_iterations_per_node :  %s\nrestrict_boundary_nodes_along_surface :  %s\nn_attempts :  %s" % (self._quality_measure, self._target_quality, self._dihedral_angle, self._n_iterations_per_node, self._restrict_boundary_nodes_along_surface, self._n_attempts)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def quality_measure(self) -> CellQualityMeasure:
        """Specify cell quality measure to be used for volume mesh improvement. The default value for cell quality measure is skewness.
        """
        return self._quality_measure

    @quality_measure.setter
    def quality_measure(self, value: CellQualityMeasure):
        self._quality_measure = value

    @property
    def target_quality(self) -> float:
        """Specify target quality used for the mesh improvement based on specified quality measure.
        """
        return self._target_quality

    @target_quality.setter
    def target_quality(self, value: float):
        self._target_quality = value

    @property
    def dihedral_angle(self) -> float:
        """Dihedral angle used to mantain features of boundary face zonelets.
        """
        return self._dihedral_angle

    @dihedral_angle.setter
    def dihedral_angle(self, value: float):
        self._dihedral_angle = value

    @property
    def n_iterations_per_node(self) -> int:
        """Number of iterations per node to be moved.
        """
        return self._n_iterations_per_node

    @n_iterations_per_node.setter
    def n_iterations_per_node(self, value: int):
        self._n_iterations_per_node = value

    @property
    def restrict_boundary_nodes_along_surface(self) -> bool:
        """Option to restrict the movement of the boundary node to the plane containing the boundary faces sharing the boundary node.
        """
        return self._restrict_boundary_nodes_along_surface

    @restrict_boundary_nodes_along_surface.setter
    def restrict_boundary_nodes_along_surface(self, value: bool):
        self._restrict_boundary_nodes_along_surface = value

    @property
    def n_attempts(self) -> int:
        """Number of attempts to improve specified quality measure by node movement.
        """
        return self._n_attempts

    @n_attempts.setter
    def n_attempts(self, value: int):
        self._n_attempts = value

class CheckMeshParams(CoreObject):
    """Parameters used to check mesh.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CheckMeshParams.

        Parameters
        ----------
        model: Model
            Model to create a CheckMeshParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a CheckMeshParams object with provided parameters.

        Examples
        --------
        >>> check_mesh_params = prime.CheckMeshParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CheckMeshParams")
                    json_data = param_json["CheckMeshParams"] if "CheckMeshParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of CheckMeshParams.

        """
        args = locals()
        [CheckMeshParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CheckMeshParams.

        Examples
        --------
        >>> CheckMeshParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CheckMeshParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class VolumeMeshToolResults(CoreObject):
    """Result associated with the volume mesh tool operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode):
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeMeshToolResults.

        Parameters
        ----------
        model: Model
            Model to create a VolumeMeshToolResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the volume mesh tool operation.
        json_data: dict, optional
            JSON dictionary to create a VolumeMeshToolResults object with provided parameters.

        Examples
        --------
        >>> volume_mesh_tool_results = prime.VolumeMeshToolResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["error_code"] if "error_code" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [error_code])
            if all_field_specified:
                self.__initialize(
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeMeshToolResults")
                    json_data = param_json["VolumeMeshToolResults"] if "VolumeMeshToolResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( VolumeMeshToolResults._default_params["error_code"] if "error_code" in VolumeMeshToolResults._default_params else ErrorCode(json_data["error_code"] if "error_code" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None):
        """Set the default values of VolumeMeshToolResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the volume mesh tool operation.
        """
        args = locals()
        [VolumeMeshToolResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeMeshToolResults.

        Examples
        --------
        >>> VolumeMeshToolResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeMeshToolResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["error_code"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s" % (self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the volume mesh tool operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class CheckMeshResults(CoreObject):
    """Result associated with the check mesh operation.
    """
    _default_params = {}

    def __initialize(
            self,
            has_non_positive_volumes: bool,
            has_non_positive_areas: bool,
            has_invalid_shape: bool,
            has_left_handed_faces: bool,
            error_code: ErrorCode,
            warning_codes: List[WarningCode]):
        self._has_non_positive_volumes = has_non_positive_volumes
        self._has_non_positive_areas = has_non_positive_areas
        self._has_invalid_shape = has_invalid_shape
        self._has_left_handed_faces = has_left_handed_faces
        self._error_code = ErrorCode(error_code)
        self._warning_codes = warning_codes

    def __init__(
            self,
            model: CommunicationManager=None,
            has_non_positive_volumes: bool = None,
            has_non_positive_areas: bool = None,
            has_invalid_shape: bool = None,
            has_left_handed_faces: bool = None,
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CheckMeshResults.

        Parameters
        ----------
        model: Model
            Model to create a CheckMeshResults object with default parameters.
        has_non_positive_volumes: bool, optional
            Indicates whether mesh has non positive volumes.
        has_non_positive_areas: bool, optional
            Indicates whether mesh has non positive areas.
        has_invalid_shape: bool, optional
            Indicates whether mesh has invalid shape.
        has_left_handed_faces: bool, optional
            Indicates whether mesh has left handed faces.
        error_code: ErrorCode, optional
            Error code associated with the check grid operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the check grid operation.
        json_data: dict, optional
            JSON dictionary to create a CheckMeshResults object with provided parameters.

        Examples
        --------
        >>> check_mesh_results = prime.CheckMeshResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["hasNonPositiveVolumes"] if "hasNonPositiveVolumes" in json_data else None,
                json_data["hasNonPositiveAreas"] if "hasNonPositiveAreas" in json_data else None,
                json_data["hasInvalidShape"] if "hasInvalidShape" in json_data else None,
                json_data["hasLeftHandedFaces"] if "hasLeftHandedFaces" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [has_non_positive_volumes, has_non_positive_areas, has_invalid_shape, has_left_handed_faces, error_code, warning_codes])
            if all_field_specified:
                self.__initialize(
                    has_non_positive_volumes,
                    has_non_positive_areas,
                    has_invalid_shape,
                    has_left_handed_faces,
                    error_code,
                    warning_codes)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CheckMeshResults")
                    json_data = param_json["CheckMeshResults"] if "CheckMeshResults" in param_json else {}
                    self.__initialize(
                        has_non_positive_volumes if has_non_positive_volumes is not None else ( CheckMeshResults._default_params["has_non_positive_volumes"] if "has_non_positive_volumes" in CheckMeshResults._default_params else (json_data["hasNonPositiveVolumes"] if "hasNonPositiveVolumes" in json_data else None)),
                        has_non_positive_areas if has_non_positive_areas is not None else ( CheckMeshResults._default_params["has_non_positive_areas"] if "has_non_positive_areas" in CheckMeshResults._default_params else (json_data["hasNonPositiveAreas"] if "hasNonPositiveAreas" in json_data else None)),
                        has_invalid_shape if has_invalid_shape is not None else ( CheckMeshResults._default_params["has_invalid_shape"] if "has_invalid_shape" in CheckMeshResults._default_params else (json_data["hasInvalidShape"] if "hasInvalidShape" in json_data else None)),
                        has_left_handed_faces if has_left_handed_faces is not None else ( CheckMeshResults._default_params["has_left_handed_faces"] if "has_left_handed_faces" in CheckMeshResults._default_params else (json_data["hasLeftHandedFaces"] if "hasLeftHandedFaces" in json_data else None)),
                        error_code if error_code is not None else ( CheckMeshResults._default_params["error_code"] if "error_code" in CheckMeshResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        warning_codes if warning_codes is not None else ( CheckMeshResults._default_params["warning_codes"] if "warning_codes" in CheckMeshResults._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            has_non_positive_volumes: bool = None,
            has_non_positive_areas: bool = None,
            has_invalid_shape: bool = None,
            has_left_handed_faces: bool = None,
            error_code: ErrorCode = None,
            warning_codes: List[WarningCode] = None):
        """Set the default values of CheckMeshResults.

        Parameters
        ----------
        has_non_positive_volumes: bool, optional
            Indicates whether mesh has non positive volumes.
        has_non_positive_areas: bool, optional
            Indicates whether mesh has non positive areas.
        has_invalid_shape: bool, optional
            Indicates whether mesh has invalid shape.
        has_left_handed_faces: bool, optional
            Indicates whether mesh has left handed faces.
        error_code: ErrorCode, optional
            Error code associated with the check grid operation.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the check grid operation.
        """
        args = locals()
        [CheckMeshResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CheckMeshResults.

        Examples
        --------
        >>> CheckMeshResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CheckMeshResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._has_non_positive_volumes is not None:
            json_data["hasNonPositiveVolumes"] = self._has_non_positive_volumes
        if self._has_non_positive_areas is not None:
            json_data["hasNonPositiveAreas"] = self._has_non_positive_areas
        if self._has_invalid_shape is not None:
            json_data["hasInvalidShape"] = self._has_invalid_shape
        if self._has_left_handed_faces is not None:
            json_data["hasLeftHandedFaces"] = self._has_left_handed_faces
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "has_non_positive_volumes :  %s\nhas_non_positive_areas :  %s\nhas_invalid_shape :  %s\nhas_left_handed_faces :  %s\nerror_code :  %s\nwarning_codes :  %s" % (self._has_non_positive_volumes, self._has_non_positive_areas, self._has_invalid_shape, self._has_left_handed_faces, self._error_code, '[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def has_non_positive_volumes(self) -> bool:
        """Indicates whether mesh has non positive volumes.
        """
        return self._has_non_positive_volumes

    @has_non_positive_volumes.setter
    def has_non_positive_volumes(self, value: bool):
        self._has_non_positive_volumes = value

    @property
    def has_non_positive_areas(self) -> bool:
        """Indicates whether mesh has non positive areas.
        """
        return self._has_non_positive_areas

    @has_non_positive_areas.setter
    def has_non_positive_areas(self, value: bool):
        self._has_non_positive_areas = value

    @property
    def has_invalid_shape(self) -> bool:
        """Indicates whether mesh has invalid shape.
        """
        return self._has_invalid_shape

    @has_invalid_shape.setter
    def has_invalid_shape(self, value: bool):
        self._has_invalid_shape = value

    @property
    def has_left_handed_faces(self) -> bool:
        """Indicates whether mesh has left handed faces.
        """
        return self._has_left_handed_faces

    @has_left_handed_faces.setter
    def has_left_handed_faces(self, value: bool):
        self._has_left_handed_faces = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the check grid operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the check grid operation.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value



"""Module for gRPC utilities."""
import ansys.meshing.prime.internals.defaults as defaults


def get_default_channel_args():
    """Get default channel arguments for gRPC.

    Returns
    -------
    List
        List with channel parameters.
    """
    return [
        # ('grpc.keepalive_time_ms', 60000), # 1 minute
        ('grpc.keepalive_timeout_ms', 10000),  # 10 seconds
        ('grpc.keepalive_permit_without_calls', True),
        ('grpc.http2.max_pings_without_data', 0),  # Disable
        ('grpc.max_send_message_length', defaults.max_message_length()),
        ('grpc.max_receive_message_length', defaults.max_message_length()),
    ]



"""Module for abstract communicator."""
from abc import abstractmethod


class Communicator(object):
    """Provides the abstract class for the server communicator."""

    @abstractmethod
    def serve(self, model, command, *args, **kwargs) -> dict:
        """Serve the model and send a command to the server.

        Parameters
        ----------
        model : Model
            Model to serve.
        command : str
            Command to send to the server.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def initialize_params(self, model, param_name: str, *args) -> dict:
        """Initialize parameters on the server side.

        Parameters
        ----------
        model : Model
            Model to initialize parameter on.
        param_name : str
            Parameter to initialize.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def run_on_server(self, model, recipe: str) -> dict:
        """Run the command on the server side.

        Parameters
        ----------
        model : Model
            Model to run commands on.
        recipe : str
            Command to run.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def import_cad(self, model, file_name: str, *args) -> dict:
        """Import a CAD file from local.

        Parameters
        ----------
        model : Model
            Model to import.
        file_name : str
            Name of the CAD file.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def close(self):
        """Close communication when deleting the instance."""
        pass



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class VolumeControlParams(CoreObject):
    """Volume control parameters are used to define the volume type in the volume control.
    """
    _default_params = {}

    def __initialize(
            self,
            cell_zonelet_type: CellZoneletType,
            skip_hexcore: bool):
        self._cell_zonelet_type = CellZoneletType(cell_zonelet_type)
        self._skip_hexcore = skip_hexcore

    def __init__(
            self,
            model: CommunicationManager=None,
            cell_zonelet_type: CellZoneletType = None,
            skip_hexcore: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeControlParams.

        Parameters
        ----------
        model: Model
            Model to create a VolumeControlParams object with default parameters.
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        skip_hexcore: bool, optional
            Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        json_data: dict, optional
            JSON dictionary to create a VolumeControlParams object with provided parameters.

        Examples
        --------
        >>> volume_control_params = prime.VolumeControlParams(model = model)
        """
        if json_data:
            self.__initialize(
                CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None),
                json_data["skipHexcore"] if "skipHexcore" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [cell_zonelet_type, skip_hexcore])
            if all_field_specified:
                self.__initialize(
                    cell_zonelet_type,
                    skip_hexcore)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeControlParams")
                    json_data = param_json["VolumeControlParams"] if "VolumeControlParams" in param_json else {}
                    self.__initialize(
                        cell_zonelet_type if cell_zonelet_type is not None else ( VolumeControlParams._default_params["cell_zonelet_type"] if "cell_zonelet_type" in VolumeControlParams._default_params else CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None)),
                        skip_hexcore if skip_hexcore is not None else ( VolumeControlParams._default_params["skip_hexcore"] if "skip_hexcore" in VolumeControlParams._default_params else (json_data["skipHexcore"] if "skipHexcore" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            cell_zonelet_type: CellZoneletType = None,
            skip_hexcore: bool = None):
        """Set the default values of VolumeControlParams.

        Parameters
        ----------
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        skip_hexcore: bool, optional
            Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        """
        args = locals()
        [VolumeControlParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeControlParams.

        Examples
        --------
        >>> VolumeControlParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeControlParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._cell_zonelet_type is not None:
            json_data["cellZoneletType"] = self._cell_zonelet_type
        if self._skip_hexcore is not None:
            json_data["skipHexcore"] = self._skip_hexcore
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "cell_zonelet_type :  %s\nskip_hexcore :  %s" % (self._cell_zonelet_type, self._skip_hexcore)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def cell_zonelet_type(self) -> CellZoneletType:
        """Cell zonelet type is used to define the type of the associated volume.
        """
        return self._cell_zonelet_type

    @cell_zonelet_type.setter
    def cell_zonelet_type(self, value: CellZoneletType):
        self._cell_zonelet_type = value

    @property
    def skip_hexcore(self) -> bool:
        """Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        """
        return self._skip_hexcore

    @skip_hexcore.setter
    def skip_hexcore(self, value: bool):
        self._skip_hexcore = value



"""Implementation of JSOnEncoder that can handle byte arrays."""
import re

import numpy as np

ESCAPE = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t]')
ESCAPE_ASCII = re.compile(r'([\\"]|[^\ -~])')
HAS_UTF8 = re.compile(b'[\x80-\xff]')
ESCAPE_DCT = {
    '\\': '\\\\',
    '"': '\\"',
    '\b': '\\b',
    '\f': '\\f',
    '\n': '\\n',
    '\r': '\\r',
    '\t': '\\t',
}
for i in range(0x20):
    ESCAPE_DCT.setdefault(chr(i), '\\u{0:04x}'.format(i))
    # ESCAPE_DCT.setdefault(chr(i), '\\u%04x' % (i,))

INFINITY = float('inf')

NUMPY_TYPEMAP = {
    np.int8: 0x11,
    np.int16: 0x12,
    np.int32: 0x13,
    np.int64: 0x14,
    np.uint8: 0x19,
    np.uint16: 0x1A,
    np.uint32: 0x1B,
    np.uint64: 0x1C,
    np.float32: 0x21,
    np.float64: 0x22,
}


def _get_numpy_dtype(current_dtype):
    if np.issubdtype(current_dtype, np.int8):
        return np.int8
    elif np.issubdtype(current_dtype, np.int16):
        return np.int16
    elif np.issubdtype(current_dtype, np.int32):
        return np.int32
    elif np.issubdtype(current_dtype, np.int64):
        return np.int64
    elif np.issubdtype(current_dtype, np.uint8):
        return np.uint8
    elif np.issubdtype(current_dtype, np.uint16):
        return np.uint16
    elif np.issubdtype(current_dtype, np.uint32):
        return np.uint32
    elif np.issubdtype(current_dtype, np.uint64):
        return np.uint64
    elif np.issubdtype(current_dtype, np.float32):
        return np.float32
    elif np.issubdtype(current_dtype, np.float64):
        return np.float64
    else:
        raise ValueError(f'Unsupported dtype {current_dtype}for ' f'numpy array optimizaation')


def _check_if_byteorder_needs_to_change(dtype_: np.dtype):
    """Check if we need to change the byteorder of a numpy array.

    The wire format for numpy arrays is always in bigendian to ensure
    cross-platform compatibility. Hence, this check ensures that we switch the
    dtype of an ndarray only when the existing dtype does not have a byteorder
    defined or if the defined byteorder is little-endian.
    """
    import sys

    if dtype_.byteorder == '<' or dtype_.byteorder == '|':
        return True
    if dtype_.byteorder == '=' and sys.byteorder == 'little':
        return True
    return False


def py_encode_basestring(s):
    """Return a bytearray JSON representation of a Python string."""

    def replace(match):
        return ESCAPE_DCT[match.group(0)]

    return str('"' + ESCAPE.sub(replace, s) + '"').encode('utf-8')


encode_basestring = py_encode_basestring


def py_encode_basestring_ascii(s):
    """Return an ASCII-only bytearray JSON representation of a Python string."""

    def replace(match):
        s = match.group(0)
        try:
            return ESCAPE_DCT[s]
        except KeyError:
            n = ord(s)
            if n < 0x10000:
                return '\\u{0:04x}'.format(n)
                # return '\\u%04x' % (n,)
            else:
                # surrogate pair
                n -= 0x10000
                s1 = 0xD800 | ((n >> 10) & 0x3FF)
                s2 = 0xDC00 | (n & 0x3FF)
                return '\\u{0:04x}\\u{1:04x}'.format(s1, s2)

    return str('"' + ESCAPE_ASCII.sub(replace, s) + '"').encode('utf-8')


encode_basestring_ascii = py_encode_basestring_ascii


class JSONEncoder:
    """Class to encode Object to JSON format.

    Parameters
    ----------
    skipkeys : bool, optional
        Skip keys, by default False.
    ensure_ascii : bool, optional
        Ensure that we use ASCII characters, by default True.
    check_circular : bool, optional
        Check circular loops, by default True.
    allow_nan : bool, optional
        Allow not number values, by default True.
    sort_keys : bool, optional
        Order the keys, by default False.
    separators : Any, optional
        String separators, by default None.
    default : Any, optional
        Default error message, by default None.
    """

    item_separator = b','
    key_separator = b':'

    def __init__(
        self,
        *,
        skipkeys=False,
        ensure_ascii=True,
        check_circular=True,
        allow_nan=True,
        sort_keys=False,
        separators=None,
        default=None,
    ):
        """Initialize of JSON Encoder."""
        self.skipkeys = skipkeys
        self.ensure_ascii = ensure_ascii
        self.check_circular = check_circular
        self.allow_nan = allow_nan
        self.sort_keys = sort_keys
        if separators is not None:
            self.item_separator, self.key_separator = separators
        if default is not None:
            self.default = default

    def default(self, obj):
        """Raise default error.

        Parameters
        ----------
        obj : Any
            Object that is not serializable.

        Raises
        ------
        TypeError
            Raise the error with the proper type
        """
        raise TypeError(f'Object of type {type(obj)} is not JSON serializable')

    def encode(self, obj):
        """Return a bytearray containing JSON.

        Return a bytearray containing JSON representation of Python Data
        structure, with the vector extension
        """
        # This is for extremely simple cases and benchmarks.
        if isinstance(obj, str):
            if self.ensure_ascii:
                return encode_basestring_ascii(obj)
            else:
                return encode_basestring(obj)

        chunks = self.iterencode(obj)
        if not isinstance(chunks, (list, tuple)):
            chunks = list(chunks)

        return b''.join(chunks)

    def iterencode(self, obj):
        """Encode the given object and yield the bytearray representationas available."""
        if self.check_circular:
            markers = {}
        else:
            markers = None
        if self.ensure_ascii:
            _encoder = encode_basestring_ascii
        else:
            _encoder = encode_basestring

        def floatstr(
            o, allow_nan=self.allow_nan, _repr=float.__repr__, _inf=INFINITY, _neginf=-INFINITY
        ):
            # Check for specials.  Note that this type of test is processor
            # and/or platform-specific, so do tests which don't depend on the
            # internals.

            if o != o:
                text = b'NaN'
            elif o == _inf:
                text = b'Infinity'
            elif o == _neginf:
                text = b'-Infinity'
            else:
                return _repr(o).encode('utf-8')

            if not allow_nan:
                raise ValueError("Out of range float values are not JSON compliant: " + repr(o))

            return text

        def intstr(o, _repr=int.__repr__):
            return _repr(o).encode('utf-8')

        def vectorstr(o: np.ndarray, _map_dtype=NUMPY_TYPEMAP):
            dtype_: np.dtype = _get_numpy_dtype(o.dtype)
            value_type = _map_dtype.get(dtype_)
            buffer = b'<'
            buffer += int(o.size * o.itemsize).to_bytes(8, byteorder='big')
            buffer += _map_dtype.get(dtype_).to_bytes(1, byteorder='big')
            if _check_if_byteorder_needs_to_change(o.dtype):
                buffer += o.astype(o.dtype.newbyteorder('>')).tobytes()
            else:
                buffer += o.tobytes()
            buffer += b'>'
            return buffer

        _iterencode = _make_iterencode(
            markers,
            self.default,
            _encoder,
            floatstr,
            self.key_separator,
            self.item_separator,
            self.sort_keys,
            self.skipkeys,
            intstr,
            vectorstr,
        )
        return _iterencode(obj)


def _make_iterencode(
    markers,
    _default,
    _encoder,
    _floatstr,
    _key_separator,
    _item_separator,
    _sort_keys,
    _skipkeys,
    _intstr,
    _vectorstr,
):
    def _iterencode_list(lst):
        if not lst:
            yield b'[]'
            return
        if markers is not None:
            markerid = id(lst)
            if markerid in markers:
                raise ValueError("Circular reference detected")
            markers[markerid] = lst
        buf = bytearray()
        buf += b'['
        separator = _item_separator
        first = True
        for value in lst:
            if first:
                first = False
            else:
                buf = separator
            if isinstance(value, str):
                yield buf + _encoder(value)
            elif value is None:
                yield buf + b'null'
            elif value is True:
                yield buf + b'true'
            elif value is False:
                yield buf + b'false'
            elif isinstance(value, int):
                # Subclasses of int/float may override __repr__, but we still
                # want to encode them as integers/floats in JSON. One example
                # within the standard library is IntEnum.
                yield buf + _intstr(value)
            elif isinstance(value, float):
                # see comment above for int
                yield buf + _floatstr(value)
            else:
                yield buf
                if isinstance(value, (list, tuple)):
                    chunks = _iterencode_list(value)
                elif isinstance(value, dict):
                    chunks = _iterencode_dict(value)
                else:
                    chunks = _iterencode(value)
                yield from chunks
        yield b']'
        if markers is not None:
            del markers[markerid]

    def _iterencode_dict(dct):
        if not dct:
            yield b'{}'
            return
        if markers is not None:
            markerid = id(dct)
            if markerid in markers:
                raise ValueError("Circular reference detected")
            markers[markerid] = dct
        yield b'{'
        item_separator = _item_separator
        first = True
        if _sort_keys:
            items = sorted(dct.items())
        else:
            items = dct.items()
        for key, value in items:
            if isinstance(key, str):
                pass
            # JavaScript is weakly typed for these, so it makes sense to
            # also allow them.  Many encoders seem to do something like this.
            elif isinstance(key, float):
                # see comment for int/float in _make_iterencode
                key = _floatstr(key)
            elif key is True:
                key = b'true'
            elif key is False:
                key = b'false'
            elif key is None:
                key = b'null'
            elif isinstance(key, int):
                # see comment for int/float in _make_iterencode
                key = _intstr(key)
            elif _skipkeys:
                continue
            else:
                raise TypeError(
                    f'keys must be str, int, float, bool or None, ' f'not {key.__class__.__name__}'
                )
            if first:
                first = False
            else:
                yield item_separator
            yield _encoder(key)
            yield _key_separator
            if isinstance(value, str):
                yield _encoder(value)
            elif value is None:
                yield b'null'
            elif value is True:
                yield b'true'
            elif value is False:
                yield b'false'
            elif isinstance(value, int):
                # see comment for int/float in _make_iterencode
                yield _intstr(value)
            elif isinstance(value, float):
                # see comment for int/float in _make_iterencode
                yield _floatstr(value)
            else:
                if isinstance(value, (list, tuple)):
                    chunks = _iterencode_list(value)
                elif isinstance(value, dict):
                    chunks = _iterencode_dict(value)
                else:
                    chunks = _iterencode(value)
                yield from chunks
        yield b'}'
        if markers is not None:
            del markers[markerid]

    def _iterencode(o):
        if isinstance(o, str):
            yield _encoder(o)
        elif o is None:
            yield b'null'
        elif o is True:
            yield b'true'
        elif o is False:
            yield b'false'
        elif isinstance(o, int):
            # see comment for int/float in _make_iterencode
            yield _intstr(o)
        elif isinstance(o, float):
            # see comment for int/float in _make_iterencode
            yield _floatstr(o)
        elif isinstance(o, np.ndarray):
            yield _vectorstr(o)
        elif isinstance(o, (list, tuple)):
            yield from _iterencode_list(o)
        elif isinstance(o, dict):
            yield from _iterencode_dict(o)
        else:
            if markers is not None:
                markerid = id(o)
                if markerid in markers:
                    raise ValueError("Circular reference detected")
                markers[markerid] = o
            o = _default(o)
            yield from _iterencode(o)
            if markers is not None:
                del markers[markerid]

    return _iterencode
